local cartridge = require('cartridge')
local yaml = require("yaml")
local checks = require('checks')
local sbroad_router = require('sbroad.router')

_G.query = nil
_G.explain = nil
_G.calculate_bucket_id = nil
_G.calculate_bucket_id_by_dict = nil
_G.sql_execute = nil
_G.set_schema = nil

local function explain(query)
    return sbroad_router.explain(query)
end

local function query(q, params)
    return sbroad_router.query_execute(q, params)
end

local function calculate_bucket_id(space_name, values)
    checks('string', 'table')
    local shard_key = cartridge.config_get_deepcopy().schema.spaces[space_name].sharding_key

    local shard_val = ''
    for _, key in ipairs(shard_key) do
        shard_val = shard_val .. tostring(values[key])
    end

    local bucket_id = sbroad_router.calculate_bucket_id(shard_val)
    return bucket_id
end

local function calculate_bucket_id_by_dict(space_name, values) -- luacheck: no unused args
    return sbroad_router.calculate_bucket_id_by_dict(space_name, values)
end

local function set_schema(new_schema)
    checks('table|string')

    local schema_str = new_schema
    if type(new_schema) == 'table' then
        schema_str = yaml.encode(new_schema)
    end
    local _, err = cartridge.set_schema(schema_str)
    if err ~= nil then
        return err
    end

    return nil
end

local function init(opts) -- luacheck: no unused args
    _G.query = query
    _G.explain = explain
    _G.calculate_bucket_id = calculate_bucket_id
    _G.calculate_bucket_id_by_dict = calculate_bucket_id_by_dict
    _G.sql_execute = query
    _G.set_schema = set_schema

    sbroad_router.init()

    return true
end

local function stop()
    return true
end

local function validate_config(conf_new, conf_old) -- luacheck: no unused args
    return true
end

local function apply_config(conf, opts) -- luacheck: no unused args
    sbroad_router.invalidate_cache()
    return true
end

return {
    role_name = 'app.roles.api',
    init = init,
    stop = stop,
    validate_config = validate_config,
    apply_config = apply_config,
    dependencies = {'cartridge.roles.vshard-router'},
}
